# **********************************************************************
#                   rtl_power_gnuplot.tcl
#                   =====================
#                     Uwe Berger; 2014
#
# Das Tcl-Script generiert aus einer rtl_power-Ausgabedatei ein
# Heatmap-Diagramm mit Hilfe des Tools gnuplot (siehe auch unten
# "Steuerung").
#
# Aufruf:
# -------
# tclsh rtl_power_gnuplot.tcl <inputfile>
#
# <inputfile> --> csv-Datei aus rtl_power
#
#
# Format csv-Datei von rtl_power:
# -------------------------------
# 0      1        2         3         4           5          6   7   ...
# datum, uhrzeit, min-freq, max-freq, delta-freq, fft-count, db, db, ...
#
#
# Steuerung:
# ----------
# pipe_to_gnuplot=0 -> es wird eine Kommando-Inputdatei fuer
#                      gnuplot generiert und die gnuplot-Datendatei
#                      bleibt erhalten
# pipe_to_gnuplot=1 -> gnuplot-Kommandos werden direkt zu gnuplot
#                      via Pipe geschickt; gnuplot-Datendatei wird
#                      nach Generierung der Heatmap geloescht
#
#
# Konvention Dateinamen:
# ----------------------
# Saemtliche generierten Dateien (gnuplot-Daten-/-Kommando-Dateien und
# Heatmap-Bilddatei) haben als Dateinamen-Bestandteil folgende
# Informationen aus den Daten der Ursprungsdatei:
# * untere Grenze Frequenz (minfreq)
# * obere Grenze Frequenz (maxfreq)
# * kleinster Abtastzeitpunkt (mints)
# * groesster Abtastzeitpunkt (maxts)
#
# xxx_minfreq-maxfreq_mints-maxts.ext
#
#
# Beispielaufruf rtl_power
# ------------------------
# rtl_power -f 433M:435M:10k -i 2s -e 15m ism_433.csv
#
#
# ---------
# Have fun!
#
# **********************************************************************

# gnuplot-Kommandodateien oder direkt via Pipe (siehe oben!)
set pipe_to_gnuplot		1

# Groesse des Diagramms (einschiesslich Beschriftung etc.)
set plot_xsize			1200
set plot_ysize			800

# Skaleneinteilung x-/y-Achse (autofreq --> gnuplot berechnet selbst...) 
set ytics "autofreq"
set xtics "autofreq"

# ein paar globale Variable
set rtl_power_file 		""
set gnuplot_data_file	"gnuplot_data.txt"
set heatmap_file		""
set heatmap_pic_format  "png"
set gp_min_freq			10000000000
set gp_max_freq			0
set gp_min_timestamp	"9999-12-31T23:59:59"
set gp_max_timestamp	"1970-01-01T00:00:00"
set freq_steps			""
set time_steps			"42"


# *****************************
# GNUPLOT-Daten-Datei erzeugen
# *****************************
proc transform_csv {} {
	global gnuplot_cmd_file gp_min_freq gp_max_freq
	global gp_min_timestamp gp_max_timestamp freq_steps time_steps
	global heatmap_pic_format pipe_to_gnuplot heatmap_file
	global rtl_power_file gnuplot_data_file 
	puts "transform: $rtl_power_file -> $gnuplot_data_file"
	set fd1 [open $rtl_power_file r]
	set fd2 [open $gnuplot_data_file w]
	set t0 ""
	set t1 ""
	while {[gets $fd1 line] >= 0} {
		# Leerzeichen eleminieren
		set line [string map {" " ""} $line]
		# an Kommatas separieren
		set line [split $line ,]
		# Timestamp zusammensetzen 
		set date [lindex $line 0]
		set time [lindex $line 1]
		set timestamp "$date\T$time"
		# ...fuer Berechnung Zeitdifferenz...
		if {![string equal $t1 $timestamp]} {
			set t0 $t1
			set t1 $timestamp
		}
		if {$gp_min_timestamp > $timestamp} {set gp_min_timestamp $timestamp}
		if {$gp_max_timestamp < $timestamp} {set gp_max_timestamp $timestamp}
		# Werte-Trippel generieren (Timestamp, Frequenz, DB-Wert)
		set i 0
		set min_freq [lindex $line 2]
		set max_freq [lindex $line 3]
		if {$gp_min_freq > $min_freq} {set gp_min_freq $min_freq}
		if {$gp_max_freq < $max_freq} {set gp_max_freq $max_freq}
		set freq_steps [lindex $line 4]
		set line [lrange $line 6 end-1]
		foreach db $line {
			set freq_db [expr $min_freq + $i * $freq_steps]
			if {$freq_db < $max_freq} {
				puts $fd2 "$timestamp $freq_db $db";
			}
			incr i
		}
	}
	close $fd2
	close $fd1
	# Zeitdifferenz berechnen
	set time_steps [expr [clock scan $t1 -format "%Y-%m-%dT%H:%M:%S"]\
	                   - [clock scan $t0 -format "%Y-%m-%dT%H:%M:%S"]]
	# noch ein paar Dateinamen generieren...
	set min_ts [string map {- "" : "" T ""} $gp_min_timestamp]
	set max_ts [string map {- "" : "" T ""} $gp_max_timestamp]
	set filename_component "$gp_min_freq\-$gp_max_freq\_$min_ts\-$max_ts"
	set heatmap_file "heatmap_$filename_component.$heatmap_pic_format"
	if {!$pipe_to_gnuplot} {
		set gnuplot_cmd_file "gnuplot_cmd_$filename_component.txt"
		set new_gnuplot_data_file "gnuplot_data_$filename_component.txt"
		puts "rename: $gnuplot_data_file -> $new_gnuplot_data_file"
		file rename -force -- $gnuplot_data_file $new_gnuplot_data_file
		set gnuplot_data_file $new_gnuplot_data_file
	}
}

# ******************************************
# GNUPLOT-Kommando-Datei erzeugen bzw. Pipe
# ******************************************
proc generate_heatmap {} {
	global gp_min_freq gp_max_freq gp_min_timestamp gp_max_timestamp 
	global heatmap_pic_format pipe_to_gnuplot
	global heatmap_file gnuplot_data_file gnuplot_cmd_file
	global plot_xsize plot_ysize ytics xtics freq_steps time_steps
	if {$pipe_to_gnuplot} {
		puts "generate heatmap: $heatmap_file"
	} else {
		puts "generate components for heatmap: $gnuplot_data_file, $gnuplot_cmd_file"
	}
	set  pid [open $gnuplot_cmd_file w]
	puts $pid "set timefmt '%Y-%m-%dT%H:%M:%S'"
	puts $pid "set view map"
	puts $pid "set terminal $heatmap_pic_format font arial 10 size $plot_xsize, $plot_ysize"
	puts $pid "set output '$heatmap_file'"
	puts $pid "set key off"
	puts $pid "set title \"Frequency spectrum: $gp_min_freq\Hz - $gp_max_freq\Hz\\n(freq.steps: $freq_steps\Hz; timesteps: $time_steps\s)\""
	puts $pid "set xlabel 'Date/Time'"
	puts $pid "set ylabel 'Frequency \[Hz\]'"
	puts $pid "set cblabel 'Intensity \[db\]'"
	puts $pid "set xdata time"
	puts $pid "set xrange \[\"$gp_min_timestamp\":\"$gp_max_timestamp\"\]"
	puts $pid "set format x \"%m/%d\\n%H:%M\""
	puts $pid "set format y \"%1.0f\""
	puts $pid "set yrange \[$gp_min_freq:$gp_max_freq\]"
	puts $pid "set ytics $ytics out"
	puts $pid "set xtics $xtics out"
	puts $pid "plot '$gnuplot_data_file' using 1:2:3 with image"
	puts $pid ""
	close $pid
	if {$pipe_to_gnuplot} {
		file delete $gnuplot_data_file
	}
}


# ********************************
# ********************************
# ********************************

# Script-Uebergabeparameter
if {[llength $argv] != 1} {
	puts "usage: $argv0 <inputfile>"
	exit
}
set rtl_power_file [lindex $argv 0]

# gnuplot-Kommandodateien oder via Pipe
if {$pipe_to_gnuplot} {
	set gnuplot_cmd_file	"| gnuplot"
} else {
	set gnuplot_cmd_file	"gnuplot_cmd.txt"
}

# CSV-Imputdaten aufbereiten
transform_csv

# Heatmap erzeugen
generate_heatmap

	
